
;
;   showcont.inc
;
;   This file includes routines to retrieve 
;   radio data and to show the contained content
;
;   last modified: 06/07/2002 mmddyy 02:37 by Burkart
;
; List of routines:
;
; - call retrieve_radio_data
; - call show_content
;


;-----------------------------------------------
; call retrieve_radio_data
;-----------------------------------------------
; retrieve radio data from uC and store it at
; buf_radio_in_data
;-----------------------------------------------
; input: var_radio_in_data_len
;        var_radio_in_content_id
;-----------------------------------------------
; modified: ?
;-----------------------------------------------
; stack: ? + caller address
;-----------------------------------------------

retrieve_radio_data:
        call    _clrLCD                         ; clear LCD
        ld      hl, $0603                       ; move cursor
        ld      (_curPos), hl
        ld      hl, str_receiving               ; display "Receiving..."
        call    _puts

        ld      hl, var_radio_in_data_len       ; get length into de
        call    _ldhlind ; ld hl, (hl)
        ex      de, hl

        ; send <TI request radio packet>:

        call    send_start                      ; send START
        ld      hl, tiucpacket_ti_request_radio_packet ; send header
        ld      b, 3
        call    send_packet_to_uc
        xor     a                               ; send offset
        call    send_byte_to_uc
        call    send_byte_to_uc
        ld      a, e                            ; send length
        call    send_byte_to_uc
        ld      a, d
        call    send_byte_to_uc

        ; wait for <C send radio packet>:
        
        call    wait_for_start_with_timeout     ; receive START
        jp      c, error_start_timedout         ; DEVELOPMENT
        call    receive_packet                  ; receive C packet

        ld      hl, buf_received_packet+2       ; 3rd byte has to be $54
        ld      a, $54
        cp      (hl)
        jp      nz, error_packets_differ

        push    hl
        push    de

        ; send <TI ACK>:

        call    send_start                      ; send START
        ld      hl, tiucpacket_ack              ; template packet
        ld      b, 3                            ; consists of 3 bytes
        call    send_packet_to_uc               ; send to C

        pop     de
        pop     hl

        ; data at buf_received_packet:
        ;        1 word length
        ;        1 byte $54
        ;        1 byte Content-ID
        ; length-2 byte data

        inc     hl                              ; store Content-ID
        ld      a, (hl)
        ld      (var_radio_in_content_id), a
        
        inc     hl                              ; store data at buf_radio_in_data
        push    de
        pop     bc
        ld      de, buf_radio_in_data
        ldir

        ret                                     ; RETurn to caller

;-----------------------------------------------
; call show_content
;-----------------------------------------------
; Evalutate Content-ID and display according
; content. If flag_new_radio_packet == true then
; an initialization is performed otherwise only
; an update due to a keypress is done.
;-----------------------------------------------
; input: var_flag_new_radio_packet
;        var_last_key
;        var_radio_in_content_id
;        buf_radio_in_data
;-----------------------------------------------
; modified: ?
;-----------------------------------------------
; stack: ? + caller address
;-----------------------------------------------

show_content:
        ; examine Content-ID:

        ld      a, (var_radio_in_content_id)
        or      a                               ; Content-ID 0 => nothing -> RETurn to caller
        ret     z
        cp      11                              ; plain text?
        jp      z, show_content_simple_text
        cp      14                              ; fullscreen menu?
        jp      z, show_content_fullscreen_menu
        cp      15                              ; Eingabefeld?
        jp      z, show_content_eingabefeld

        ; [[[TODO all data Content-IDs]]]

        ; unsupported Content-ID:

        call    _clrLCD                         ; display "unsupported Content-ID: xxh"
        ld      hl, $0003
        ld      (_curPos), hl
        ld      hl, str_unsupported_content_id
        call    _puts
        ld      hl, var_radio_in_content_id
        call    disp_hex
        ld      a, 'h'
        call    _putc

show_content_send_client_abort:
        ; send {client abort}:

        call    send_start                      ; send START to TI
        ld      a, 2                            ; send length
        call    send_byte_to_uc
        ld      a, 0
        call    send_byte_to_uc
        ld      a, $40                          ; send packet identifier
        call    send_byte_to_uc
        ld      a, 2                            ; send Content-ID 2 = {client abort}
        call    send_byte_to_uc

       ;jr      show_content_rcv_ack            ; wait for <C ACK> and so on

show_content_rcv_ack:
        ; receive <C ACK>:

        call    wait_for_start_with_timeout     ; wait for START
        call    receive_packet                  ; receive packet
        ld      hl, tiucpacket_ack              ; check for <C ACK>
        ld      b, 3                            ; the first 3 byte have to be identical
        call    check_received_packet           ; perform the comparison
        jp      c, error_packets_differ         ; DEVELOPMENT

        call    _clrLCD                         ; clear LCD
        ld      hl, $0703                       ; move cursor
        ld      (_curPos), hl
        ld      hl, str_sending                 ; display "Sending..."
        call    _puts

        xor     a                               ; Content-ID := 0
        ld      (var_radio_in_content_id), a
;       cpl                                     ; flag_new_radio_packet := false;
;       ld      (var_flag_new_radio_packet), a

        ret                                     ; RETurn to caller

        ;---------------------------------------

show_content_simple_text:
        ld      a, (var_flag_new_radio_packet)  ; new packet?
        or      a
        jr      nz, show_content_si_text_check_key

        ; new packet -> display string:

        call    _clrLCD                         ; clear LCD
        ld      hl, $0000                       ; reset cursor
        ld      (_curPos), hl
        res     appAutoScroll, (iy+appflags)    ; disable auto scrolling; TODO: which one?
        res     textScrolled, (iy+textflags)
        ld      hl, buf_radio_in_data           ; display string
        call    _puts
        set     appAutoScroll, (iy+appflags)    ; enable auto scrolling; TODO: which one?
        set     textScrolled, (iy+textflags)

        ; now check for keypress:

show_content_si_text_check_key:
        ld      a, (var_last_key)               ; any key pressed?
        or      a
        ret     z                               ; no, return to caller

        ld      b, a                            ; store key
        call    send_start                      ; send START to C
        ld      a, 3                            ; send length
        call    send_byte_to_uc
        ld      a, 0
        call    send_byte_to_uc
        ld      a, $40                          ; send packet identifier
        call    send_byte_to_uc
        ld      a, 33                           ; send Content-ID 33 = {Plain text reply}
        call    send_byte_to_uc
        ld      a, b                            ; send key return data
        call    send_byte_to_uc

        jp      show_content_rcv_ack            ; wait for <C ACK> and so on

        ;---------------------------------------

show_content_eingabefeld:

        ld      a, (var_flag_new_radio_packet)  ; new packet?
        or      a
        jr      nz, show_content_eingabefeld_check_key

        ; new packet -> display everything:

        call    _clrLCD                         ; clear LCD
        ld      hl, buf_radio_in_data           ; move cursor
        call    _ldhlind ; ld hl, (hl)
        ld      (_curPos), hl
        res     appAutoScroll, (iy+appflags)    ; disable auto scrolling; TODO: which one?
        res     textScrolled, (iy+textflags)
        ld      hl, buf_radio_in_data+2         ; display Erluterungstext
        call    _putps
        ld      de, input2_TiML_StartPos        ; copy StartPos
        ldi
        ldi
        ldi                                     ; copy allowed chars
        ; standard code:
;       ld      de, input2_TiML_DispLen         ; copy width
;       ldi
        ; workaround for server bug: adjust DispLen to 16 max.:
        ld      a, (hl)
        inc     hl
        cp      16
        jr      c, show_content_eingabefeld_displen_ok
        ld      a, 16
show_content_eingabefeld_displen_ok:
        ld      (input2_TiML_DispLen), a
        ; end of workaround
        ld      de, input2_TiML_MaxLen          ; copy max. chars
        ldi
        ld      a, (hl)                         ; copy default text (LT-string)
        inc     a
        ld      c, a
        ld      b, 0
        ld      de, input2_Var_StrLen
        ldir
        call    input2_DisplayString            ; Eingabefeld routines
        set     appAutoScroll, (iy+appflags)    ; enable auto scrolling; TODO: which one?
        set     textScrolled, (iy+textflags)

        ; now check for keypress:

show_content_eingabefeld_check_key:
        ld      a, (var_last_key)               ; any key pressed?
        or      a
        ret     z                               ; no, return to caller

        res     appAutoScroll, (iy+appflags)    ; disable auto scrolling; TODO: which one?
        res     textScrolled, (iy+textflags)
        call    input2_key_check                ; Eingabefeld routines
        set     appAutoScroll, (iy+appflags)    ; enable auto scrolling; TODO: which one?
        set     textScrolled, (iy+textflags)

        cp      K_F2                            ; "F2" -> send {client abort}
        jp      z, show_content_send_client_abort
        cp      K_F3                            ; "F3" -> send {Eingabefeldt reply} w/ Break
        jp      z, show_content_send_eingabefeld_reply_break
        cp      K_ENTER                         ; not "ENTER" -> RETurn to caller
        ret     nz
        ; it was "ENTER" -> send {Eingabefeldt reply} w/ OK
        xor     a                               ; return "OK"
show_content_send_eingabefeld_reply_anything:
        ld      (patch_var_eingabefeld_return+1), a
        jr      show_content_send_eingabefeld_reply_doit

show_content_send_eingabefeld_reply_break:
        ld      a, 1                            ; return "Break"
        jr      show_content_send_eingabefeld_reply_anything

show_content_send_eingabefeld_reply_doit:
        call    send_start                      ; send START to TI
        ld      a, (input2_Var_StrLen)          ; calculate packet length
        add     a, 4
        call    send_byte_to_uc                 ; send length
        ld      a, 0
        call    send_byte_to_uc
        ld      a, $40                          ; send packet identifier
        call    send_byte_to_uc
        ld      a, 36                           ; send Content-ID 36 = {Eingabefeldt reply}
        call    send_byte_to_uc
patch_var_eingabefeld_return:
        ld      a, 0                            ; send return (OK / Break), see above
        call    send_byte_to_uc
        ld      hl, input2_Var_StrLen           ; send Eingabefeld data
        ld      a, (hl)
        inc     a
        ld      b, a
show_content_send_eingabefeld_reply_loop:
        ld      a, (hl)
        call    send_byte_to_uc
        inc     hl
        djnz    show_content_send_eingabefeld_reply_loop

        jp      show_content_rcv_ack            ; wait for <C ACK> and so on

        ;---------------------------------------

show_content_fullscreen_menu:
        ld      a, (var_flag_new_radio_packet)  ; new packet?
        or      a
        jr      nz, show_content_fs_menu_check_key ; no -> simply check keys

        ; new packet -> display everything:

        res     appAutoScroll, (iy+appflags)    ; disable auto scrolling; TODO: which one?
        res     textScrolled, (iy+textflags)
        ld      hl, buf_radio_in_data           ; initialize menu
        call    RunMenu
        set     appAutoScroll, (iy+appflags)    ; enable auto scrolling; TODO: which one?
        set     textScrolled, (iy+textflags)

        ; now check for keypress:

show_content_fs_menu_check_key:
        ld      a, (var_last_key)               ; any key pressed?
        or      a
        ret     z                               ; no, return to caller

        ld      hl, buf_radio_in_data           ; update menu
        call    _MenuHandLoop

        ; item in A (A=0 <=> CLEAR; A=255 <=> abort)

        cp      255                             ; return = 255 -> RETurn to caller (no item selected)
        ret     z

        cp      254                             ; return = 254 -> send {client abort}
        jp      z, show_content_send_client_abort

        ; send response to TI-Server:
        ; send <TI send radio packet>:

        ld      b, a                            ; store menu return into b
        call    send_start                      ; send START to TI
        ld      a, 3                            ; send length
        call    send_byte_to_uc
        ld      a, 0
        call    send_byte_to_uc
        ld      a, $40                          ; send packet identifier
        call    send_byte_to_uc
        ld      a, 32                           ; send Content-ID 32 = {Fullscreen menu reply}
        call    send_byte_to_uc
        ld      a, b                            ; send menu return data
        call    send_byte_to_uc

        jp      show_content_rcv_ack            ; wait for <C ACK> and so on

