;
;
;    tinet86.asm
;
;    TI-Net Client TI 86
;    (C<->TI High-Level-Protocol)
;
;
; last modified: 06/07/2002 mmddyy 02:37 by Burkart
;

#include "ti86asm.inc"

CALCTYPE = $86 ; TI-86
VERSION_HIGH = $01
VERSION_LOW = $05
; Version: 1.05

r1w1 = $C0   ; red wire 5V, white wire 5V
r0w1 = $D4   ; red wire 0V, white wire 5V
r1w0 = $E8   ; red wire 5V, white wire 0V
r0w0 = $FC   ; red wire 0V, white wire 0V

;;; _USE_DELAYS = 1                             ; use this to wait before every red/white wire transition

; TI-82 --> TI-86:

P_LINK = bPort ; linkport
_curPos = _curRow ; cursor position

.org _asm_exec_ram

        nop
        jp ProgStart
        .dw 0
        .dw ShellTitle

ShellTitle:
Str_Title:
        .db "TI-Net Client TI-86  1.05",0


ProgStart:
        ld      (EXIT_2_TIOS_old_sp), sp        ; save SP for use in EXIT_2_TIOS

        res     indicRun, (iy+indicflags)       ; clear run indicator
        res     appTextSave, (iy+appflags)      ; don't write to _textShadow
        call    _clrLCD                         ; clear LCD
        call    _homeup                         ; move cursor to upper left corner
        ld a,8                                  ; Set _winBtm back to 8, so we
        ld (_winBtm),a                          ;  can print on the last line(s)

        ld      hl, Str_Title                   ; display title
        call    _puts

#ifdef _USE_DELAYS
        call    debug_wait
#endif

        ld      a, r0w1                         ; send START
        out     (P_LINK), a

        ld      b, $80                          ; let's see (with timeout) if C replies
main_try_start_timeoutloop1:
        push    bc
        ld      b, $00
main_try_start_timeoutloop2:
        in      a, (P_LINK)                     ; white wire = 0V?
        sra     a
        sra     a
        jr      nc, main_try_start_acked        ; this would be START-ACK
        djnz    main_try_start_timeoutloop2
        pop     bc
        djnz    main_try_start_timeoutloop1

        ; timeout, C doesn't seem to be on

#ifdef _USE_DELAYS
        call    debug_wait
#endif

        ld      a, r1w1                         ; put red wire back to 5V
        out     (P_LINK), a

        call    _newline
        ld      hl, str_waiting_for_uc          ; display "Waiting for C..."
        call    _puts

        call    wait_for_start_no_timeout       ; wait for START (no timeout!)
        call    receive_packet                  ; receive packet
        ld      hl, tiucpacket_uc_re_started    ; check for <C (re)started>
        ld      b, 3                            ; the first 3 byte have to be identical

        call    check_received_packet           ; perform the comparison
        jp      c, error_packets_differ         ; DEVELOPMENT

        jr      main_send_ti_software_version   ; go on by sending <TI software version>

main_try_start_acked:
        pop     bc                              ; clear up stack (timeout variable)
        ; START was ACKed

#ifdef _USE_DELAYS
        call    debug_wait
#endif

        ld      a, r1w1                         ; put red wire back to 5V
        out     (P_LINK), a

main_try_start_wait_end:                        ; wait until both wires are 5V again (no timeout here)
        in      a, (P_LINK)
        sra     a
        sra     a
        jr      nc, main_try_start_wait_end

        ; START has successfully been sent

        ; send <TI started>:

        ld      hl, tiucpacket_ti_started       ; template packet
        ld      b, 3                            ; consists of 3 bytes
        call    send_packet_to_uc               ; send to C

        ; C should reply with <C ACK>:

        call    wait_for_start_with_timeout     ; wait for START
        jp      c, error_start_timedout         ; DEVELOPMENT

        call    receive_packet                  ; receive packet
        ld      hl, tiucpacket_ack              ; check for <C ACK>
        ld      b, 3                            ; the first 3 byte have to be identical
        call    check_received_packet           ; perform the comparison
        jp      c, error_packets_differ         ; DEVELOPMENT

main_send_ti_software_version:
        ; send <TI software version>:

        call    send_start                      ; send START

        ld      hl, tiucpacket_ti_software_version ; template packet
        ld      b, 6                            ; consists of 6 bytes
        call    send_packet_to_uc               ; send to C

        ; receive <C software version>:

        call    wait_for_start_with_timeout     ; wait for START
        jp      c, error_start_timedout         ; DEVELOPMENT

        call    receive_packet                  ; receive packet
        ld      hl, tiucpacket_uc_software_version ; check for <C software version>
        ld      b, 3                            ; the first 3 byte have to be identical
        call    check_received_packet           ; perform the comparison
        jp      c, error_packets_differ         ; DEVELOPMENT

        ; store C data (C type, SRAM size, software version):

        ld      a, (buf_received_packet+3)      ; copy C type
        ld      (var_uc_type), a
        ld      a, (buf_received_packet+4)      ; copy external SRAM size
        ld      (var_ext_sram_size), a
        ld      a, (buf_received_packet+5)      ; copy C software hi-version
        ld      (var_uc_software_hi), a
        ld      a, (buf_received_packet+6)      ; copy C software lo-version
        ld      (var_uc_software_lo), a

        ; display software versions:

        call    _clrLCD                         ; clear LCD
        ld      hl, $0002                       ; move cursor to row 3, col 0
        ld      (_curPos), hl

        ld      hl, str_this_is_a_ti_           ; display "This is a TI-xx"
        call    _puts
        ld      hl, tiucpacket_ti_software_version + 3 ; calc type
        call    disp_hex
        ld      hl, str_w_software              ; display " |w/software xx.yy|"
        call    _puts
        ld      hl, tiucpacket_ti_software_version + 4 ; ti software hi version
        call    disp_hex
        ld      a, '.'
        call    _putc
        ld      hl, tiucpacket_ti_software_version + 5 ; ti software lo version
        call    disp_hex
        ld      hl, str_connected_to_an_at89c   ; display "connected to an |AT89Cxx"
        call    _puts
        ld      hl, var_uc_type                 ; uC type
        call    disp_hex
        ld      hl, str_plus                    ; display " + xx"
        call    _puts
        ld      hl, var_ext_sram_size           ; ext. SRAM size
        call    disp_hex
        ld      hl, str_kb_ext_sram_with_software ; display "kB  |ext. SRAM with  |software xx.yy"
        call    _puts
        ld      hl, var_uc_software_hi          ; C software hi version
        call    disp_hex
        ld      a, '.'
        call    _putc
        ld      hl, var_uc_software_lo          ; C software lo version
        call    disp_hex

        ; show Eingabefeld to let user enter username

        ld      hl, $0000                       ; move cursor to row 0 col 0
        ld      (_curPos), hl
        ld      hl, str_username                ; display "Username:"
        call    _puts

        ld      hl, $0A00                       ; set Eingabefeld position
        ld      (input2_TiML_StartPos), hl
        xor     a                               ; clear default string
        ld      (input2_Var_StrLen), a
        ld      a, 8                            ; DispLen=8
        ld      (input2_TiML_DispLen), a
        ld      (input2_TiML_MaxLen), a         ; MaxLen=8
        ld      a, %11111000                    ; allowed chars: letters and numbers
        ld      (input2_TiML_AllowedChars), a

        call    input2_DisplayString            ; show Eingabefeld

main_enter_login_name:
        halt                                    ; save power
        call    _get_key                        ; get keypad data
        cp      K_F1                            ; "F1" -> send START to reset uC and RET to CrASH
        jp      z, send_start
        call    input2_key_check                ; update Eingabefeld
        cp      K_ENTER                         ; not "ENTER" -> loop again
        jr      nz, main_enter_login_name

        ; prepare username (fill empty spaces with 00h):

        ld      hl, input2_Var_StrLen           ; de = a = StrLen
        ld      a, (hl)
        ld      d, 0
        ld      e, a
        ld      a, 8                            ; a = (8-StrLen) = empty spaces
        sub     (hl)
        or      a                               ; no empty spaces?
        jr      z, main_copy_username           ; yes, simply copy all 8 characters, no prepations needed
        ld      b, a                            ; b = a = empty spaces
        ld      hl, input2_Var_String           ; hl = buffer_start + StrLen = first empty space
        add     hl, de
main_prepare_username:
        ld      (hl), $00                       ; fill space with $00
        inc     hl                              ; advance to next byte
        djnz    main_prepare_username           ; fill all empty spaces

main_copy_username:
        ; copy username:

        ld      de, tiucpacket_ti_login_data+3  ; destination: template packet
        ld      hl, input2_Var_String           ; source: Eingabefeld
        ld      bc, 8                           ; length: 8 byte
        ldir                                    ; copy it

        ; send <TI login data>:

        call    send_start                      ; send START
        ld      hl, tiucpacket_ti_login_data    ; template packet
        ld      b, 11                           ; consists of 11 bytes
        call    send_packet_to_uc               ; send to C

        ; receive <C logging in>:

        call    wait_for_start_with_timeout     ; wait for START
        jp      c, error_start_timedout         ; DEVELOPMENT

        call    receive_packet                  ; receive packet
        ld      hl, tiucpacket_uc_logging_in    ; check for <C logging in>
        ld      b, 3                            ; the first 3 byte have to be identical
        call    check_received_packet           ; perform the comparison
        jp      c, error_packets_differ         ; DEVELOPMENT

        ; send <TI ACK>:

        call    send_start                      ; send START
        ld      hl, tiucpacket_ack              ; template packet
        ld      b, 3                            ; consists of 3 bytes
        call    send_packet_to_uc               ; send to C

        ; display "Logging in...":

        call    _clrLCD                         ; clear LCD
        ld      hl, $0503                       ; move cursor to row 4 column 6
        ld      (_curPos), hl
        ld      hl, str_logging_in              ; display "Logging in..."
        call    _puts

        ; wait for <C logged in>:

        call    wait_for_start_no_timeout       ; wait for START (no timeout!)
        call    receive_packet                  ; receive packet
        ld      hl, tiucpacket_uc_logged_in     ; check for <C logged in>
        ld      b, 3                            ; the first 3 byte have to be identical
        call    check_received_packet           ; perform the comparison
        jp      c, error_packets_differ         ; DEVELOPMENT

        ; send <TI ACK>:

        call    send_start                      ; send START
        ld      hl, tiucpacket_ack              ; template packet
        ld      b, 3                            ; consists of 3 bytes
        call    send_packet_to_uc               ; send to C

        ; display "Logged in!":

        ld      hl, $0504                       ; move cursor to row 5 column 6
        ld      (_curPos), hl
        ld      hl, str_logged_in               ; display "Logged in!"
        call    _puts

        xor     a                               ; Content-ID := 0
        ld      (var_radio_in_content_id), a
        cpl                                     ; flag_new_radio_packet := false
        ld      (var_flag_new_radio_packet), a
        ld      (var_flag_clear_to_receive), a  ; flag_clear_to_receive := false



main_loop:
;       halt                                    ; wait for key while saving power
        in      a, (P_LINK)                     ; check for START
        sra     a
        call    nc, handle_main_start           ; START received -> handle it

        ld      a, (var_flag_clear_to_receive)  ; clear to receive?
        or      a
        jr      z, main_loop_clear_to_receive

        ; not clear to receive
        ld      a, (var_flag_new_radio_packet)  ; new <C received...radio...> received?
        or      a
        jr      nz, main_loop_not_ctr_check_for_logout

        ; new <C received...radio...> received
        ld      a, 255                          ; flag_new_radio_packet := false
        ld      (var_flag_new_radio_packet), a
        call    _clrLCD                         ; clear LCD
        ld      hl, $0603                       ; move cursor
        ld      (_curPos), hl
        ld      hl, str_receiving               ; display "Receiving..."
        call    _puts

main_loop_not_ctr_check_for_logout:
        call    _get_key                        ; get keypad data
        cp      K_F1                            ; "F1" -> logout
        jr      z, main_logout
        jr      main_loop                       ; another/no key -> loop again

main_loop_clear_to_receive:
        call    _get_key                        ; get keypad data
        cp      K_F1                            ; "F1" -> logout
        jr      z, main_logout
        ld      (var_last_key), a               ; store key in order to evaluate it later on

        ld      a, (var_flag_new_radio_packet)  ; new <C received...radio...> received?
        or      a
        jr      nz, main_loop_no_new_radio_data

        ; new radio data available
        call    retrieve_radio_data             ; retrieve radio packet data

main_loop_no_new_radio_data:
        call    show_content                    ; show radio packet content
        ld      a, 255                          ; flag_new_radio_packet := false
        ld      (var_flag_new_radio_packet), a
        jr      main_loop                       ; loop again
        


main_logout:
        ; send <TI exit>:

        call    send_start                      ; send START
        ld      hl, tiucpacket_ti_exit          ; template packet
        ld      b, 3                            ; consists of 3 bytes
        call    send_packet_to_uc               ; send to C

        ; receive <C ACK>:

        call    wait_for_start_with_timeout     ; wait for START
        jp      c, error_start_timedout         ; DEVELOPMENT

        call    receive_packet                  ; receive packet
        ld      hl, tiucpacket_ack              ; check for <C ACK>
        ld      b, 3                            ; the first 3 byte have to be identical
        call    check_received_packet           ; perform the comparison
        jp      c, error_packets_differ         ; DEVELOPMENT

        ; display "Logging out..."

        call    _clrLCD                         ; clear LCD
        ld      hl, $0503                       ; move cursor to row 4, col 7
        ld      (_curPos), hl
        ld      hl, str_logging_out             ; display "Logging out..."
        call    _puts

        ; receive <C logged out>:

        call    wait_for_start_no_timeout       ; wait for START (no timeout!)
        call    receive_packet                  ; receive packet
        ld      hl, tiucpacket_uc_logged_out    ; check for <C logged out>
        ld      b, 3                            ; the first 3 byte have to be identical
        call    check_received_packet           ; perform the comparison
        jp      c, error_packets_differ         ; DEVELOPMENT

        jp      main_clear_out                  ; exit to TIOS/shell

;===============================================
; TI-82 compatibility routines
;===============================================

;-----------------------------------------------
; jp EXIT_2_TIOS
;-----------------------------------------------
; exits to TIOS/shell -- the hard way...
; needs to have very old SP at (EXIT_2_TIOS_old_sp)
;-----------------------------------------------

EXIT_2_TIOS:
        ld      sp, (EXIT_2_TIOS_old_sp)        ; don't care what exactly is on the stack
        res     onInterrupt, (iy+onflags)       ; reset "ON" press
        ret                                     ; return to TIOS/shell

EXIT_2_TIOS_old_sp:
        .dw     0

;===============================================
; as a simple RET wouldn't do on an 86...
;===============================================

main_clear_out:
        res     onInterrupt, (iy+onflags)       ; reset "ON" press
        call    _clrLCD                         ; clear LCD
        call    _homeup                         ; move cursor to upper left corner
        ret                                     ; return to TIOS/shell

;===============================================
; included routines
;===============================================

#include "handleuc.inc" ; handle <C ...> packets during main loop

#include "uc-comm.inc"  ; C communication routines

#include "showcont.inc" ; retrieve and display radio data

#include "input2.inc"   ; Eingabefeld with separate init/update content
#include "timenu.inc"   ; Fullscreen menu with separate init/update content

#include "supply.inc"   ; Supply routines

;-----------------------------------------------
; Packet definitions
;-----------------------------------------------

#include "packtdef.inc" ; TI<->C packet definitions

;------------------------------------------------
; Variables & Buffers
;------------------------------------------------

#include "var_buf.inc"  ; variables & buffers

;------------------------------------------------
; Strings
;------------------------------------------------

#include "strings.inc"  ; strings

.end
